package rand

import (
	"math/rand"
	"sync"
	"time"
)

const letterBytes = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
const (
	letterIdxBits = 6                    // 6 bits to represent a letter index
	letterIdxMask = 1<<letterIdxBits - 1 // All 1-bits, as many as letterIdxBits
	letterIdxMax  = 63 / letterIdxBits   // # of letter indices fitting in 63 bits
)

var srcMutex = sync.Mutex{}
var src = rand.NewSource(time.Now().UnixNano())

// RandString returns a cryptographically-secure pseudo-random alpha-numeric string of a given length
func RandString(n int) string {
	return RandStringCharset(n, letterBytes)
}

// RandStringCharset generates, from a given charset, a cryptographically-secure pseudo-random string of a given length
func RandStringCharset(n int, charset string) string {
	srcMutex.Lock()
	defer srcMutex.Unlock()

	b := make([]byte, n)
	// A src.Int63() generates 63 random bits, enough for letterIdxMax characters!
	for i, cache, remain := n-1, src.Int63(), letterIdxMax; i >= 0; {
		if remain == 0 {
			cache, remain = src.Int63(), letterIdxMax
		}
		if idx := int(cache & letterIdxMask); idx < len(charset) {
			b[i] = charset[idx]
			i--
		}
		cache >>= letterIdxBits
		remain--
	}
	return string(b)
}
